#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "I2C.h"
#include "MPL3115A2alt.h"
#include "uart.h"
#include <string.h>
int status;

int sanity;
int whoami = 0xC4;

int OUT_P_MSB;
int OUT_P_CSB;
int OUT_P_LSB;
int OUT_T_MSB;
int OUT_T_LSB;

double current_alt;
float pressure;

void wait(void);
void UART_Write(char data);

int main(int argc, char** argv) {
    ANSELB = 0; 
    ANSELC = 0; 
    ANSELD = 0; 
    config_UART();
    /* Configure I2C */
    config_I2C();
       
    /* Initial ping */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    stopI2C1();
    
    /* Sanity check*/
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_WHOAMI);
    rstartI2C1();
    putI2C1(MPL3115A2_ADDRESS_READ);
    sanity = getI2C1(I2C_NACK);
    stopI2C1();
    
    /* Set to Altimeter with an OSR = 128 */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_CTRL_REG1);
    putI2C1(0xB8);
    stopI2C1();
    
    /* Enable Data Flags in PT_DATA_CFG */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_PT_DATA_CFG);
    putI2C1(MPL3115A2_OUT_P_DELTA_MSB);
    stopI2C1();
    
    /* Set Active */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_CTRL_REG1);
    putI2C1(0xB9);
    stopI2C1();
       
    while(1) {

        wait();
        
        /* Read STATUS Register */
        startI2C1();
        putI2C1(MPL3115A2_ADDRESS_WRITE);
        putI2C1(MPL3115A2_REGISTER_STATUS);
        rstartI2C1();
        putI2C1(MPL3115A2_ADDRESS_READ);
        status = getI2C1(I2C_NACK);
        stopI2C1();
        if (status & 0x08) {
            startI2C1();
            putI2C1(MPL3115A2_ADDRESS_WRITE);
            putI2C1(MPL3115A2_REGISTER_PRESSURE_MSB);
            rstartI2C1();
            putI2C1(MPL3115A2_ADDRESS_READ);
            OUT_P_MSB = getI2C1(I2C_NACK);
            stopI2C1();
            
            startI2C1();
            putI2C1(MPL3115A2_ADDRESS_WRITE);
            putI2C1(MPL3115A2_REGISTER_PRESSURE_CSB);
            rstartI2C1();
            putI2C1(MPL3115A2_ADDRESS_READ);
            OUT_P_CSB = getI2C1(I2C_NACK);
            stopI2C1();
            
            startI2C1();
            putI2C1(MPL3115A2_ADDRESS_WRITE);
            putI2C1(MPL3115A2_REGISTER_PRESSURE_LSB);
            rstartI2C1();
            putI2C1(MPL3115A2_ADDRESS_READ);
            OUT_P_LSB = getI2C1(I2C_NACK);
            stopI2C1();
            
            startI2C1();
            putI2C1(MPL3115A2_ADDRESS_WRITE);
            putI2C1(MPL3115A2_REGISTER_TEMP_MSB);
            rstartI2C1();
            putI2C1(MPL3115A2_ADDRESS_READ);
            OUT_T_MSB = getI2C1(I2C_NACK);
            stopI2C1();
            
            startI2C1();
            putI2C1(MPL3115A2_ADDRESS_WRITE);
            putI2C1(MPL3115A2_REGISTER_TEMP_LSB);
            rstartI2C1();
            putI2C1(MPL3115A2_ADDRESS_READ);
            OUT_T_LSB = getI2C1(I2C_NACK);
            stopI2C1();
            
            
            float pressure = (OUT_P_MSB << 8) | (OUT_P_CSB) + ((OUT_P_LSB >> 4)/16);
            //float pressure = (((OUT_P_MSB*65536) + (OUT_P_CSB*256) +(OUT_P_LSB & 0xF0))/16)/16;
           // float pressure = (OUT_T_MSB);
            char buffer[50];
            sprintf(buffer, "%f",pressure);
           
           

            putch(buffer);
            wait();
            wait();
            wait();
            wait();
            wait();
            putch("\033[2J\033[;H");
        
    }   

    //return 0;
}
}

void wait() {
    unsigned int i, j;
    for (i=0; i<20; i++){
        for (j=0; j<2500; j++) {}
    }
}

void config_UART(void){
    U1RXRbits.U1RXR = 0b0100;
    RPF0Rbits.RPF0R = 0b0011;
    
    U1MODEbits.ON = 0;          //turns it on
    U1MODEbits.UEN = 0b00;      // enables rx and tx
    U1MODEbits.PDSEL = 0;       // no parity
    U1MODEbits.STSEL = 0;       // 1 stop bit 
    int Clock = 10000000;     // the original value
    //int Clock = 7812;
    unsigned long Rate = 57600;
    //unsigned long Rate = 75;
    int Baud = (Clock/Rate/4) - 1;                 // prescaler
    U1MODEbits.BRGH = 1;        // baud rate generator prescaler 
    U1BRG = Baud;
    U1MODEbits.ON = 1; 
    U1MODEbits.UARTEN = 1;      // enables rx and tx bits
    U1STAbits.UTXEN = 1;        //enables transmit 
    U1STAbits.URXEN = 1;        // enables receive  
}

void putch(char *c) {
    int i;
    for(i=0;i<strlen(c);i++){
    UART_Write(c[i]);
   }
}


void UART_Write(char data)
{
  while(U1STAbits.UTXBF);
  U1TXREG = data;
}

/*void putch(char *f) {
    int i;
    for(i=0;i<strlen(f);i++){
       
    UART_Write(f[i]); 
    }
}*/

