/* 
 * Author: Konrad Rozanski
 */
#include <xc.h>
#include <stdio.h>
#include "MPL3115A2alt.h"
#include "I2C.h"

void config_MPL3115A2(void){
    int sanity;
    int whoami = 0xC4;
    
    /* Configure I2C */
    config_I2C();
        
    /* Initial ping */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    stopI2C1();
    
    /* Sanity check*/
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_WHOAMI);
    rstartI2C1();
    putI2C1(MPL3115A2_ADDRESS_READ);
    sanity = getI2C1(I2C_NACK);
    stopI2C1();
 
    /* Set to Altimeter with an OSR = 128 */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_CTRL_REG1);
    putI2C1(0xB8);
    stopI2C1();
    
    /* Enable Data Flags in PT_DATA_CFG */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_PT_DATA_CFG);
    putI2C1(MPL3115A2_OUT_P_DELTA_MSB);
    stopI2C1();
    
    /* Set Active */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_CTRL_REG1);
    putI2C1(0xB9);
    stopI2C1();
}

struct Temperature read_values_MPL3115A2(void){
    int status;
        
    int OUT_P_MSB;
    int OUT_P_CSB;
    int OUT_P_LSB;
    int OUT_T_MSB;
    int OUT_T_LSB;
    
    struct Temperature temp = { 0x00, 0x00};
    
    /* Read STATUS Register */
    startI2C1();
    putI2C1(MPL3115A2_ADDRESS_WRITE);
    putI2C1(MPL3115A2_REGISTER_STATUS);
    rstartI2C1();
    putI2C1(MPL3115A2_ADDRESS_READ);
    status = getI2C1(I2C_NACK);
    stopI2C1();

    if (status & 0x08) {
        startI2C1();
        putI2C1(MPL3115A2_ADDRESS_WRITE);
        putI2C1(MPL3115A2_REGISTER_PRESSURE_MSB);
        rstartI2C1();
        putI2C1(MPL3115A2_ADDRESS_READ);
        OUT_P_MSB = getI2C1(I2C_NACK);
        stopI2C1();

        startI2C1();
        putI2C1(MPL3115A2_ADDRESS_WRITE);
        putI2C1(MPL3115A2_REGISTER_PRESSURE_CSB);
        rstartI2C1();
        putI2C1(MPL3115A2_ADDRESS_READ);
        OUT_P_CSB = getI2C1(I2C_NACK);
        stopI2C1();

        startI2C1();
        putI2C1(MPL3115A2_ADDRESS_WRITE);
        putI2C1(MPL3115A2_REGISTER_PRESSURE_LSB);
        rstartI2C1();
        putI2C1(MPL3115A2_ADDRESS_READ);
        OUT_P_LSB = getI2C1(I2C_NACK);
        stopI2C1();

        startI2C1();
        putI2C1(MPL3115A2_ADDRESS_WRITE);
        putI2C1(MPL3115A2_REGISTER_TEMP_MSB);
        rstartI2C1();
        putI2C1(MPL3115A2_ADDRESS_READ);
        OUT_T_MSB = getI2C1(I2C_NACK);
        stopI2C1();

        startI2C1();
        putI2C1(MPL3115A2_ADDRESS_WRITE);
        putI2C1(MPL3115A2_REGISTER_TEMP_LSB);
       // rstartI2C1();
        putI2C1(MPL3115A2_ADDRESS_READ);
        OUT_T_LSB = getI2C1(I2C_NACK);
        stopI2C1();
        
        temp.int_part   = status;
        temp.frac_part  = OUT_T_LSB>>4;
    } 
    
    return temp;
}