/* 
 * Author: Konrad Rozanski
 */
#include "spi.h"

#define SPI_RX_IN_PROGRESS 0x0

void config_SPI(void){
    // disable interrupts during configuration
    asm volatile ("di");
    
    INTCONbits.MVEC = 0x0001;     // enable multi-vector interrupts
    
    ANSELD = 0;                   // ensure that D pins are digital
    TRAN_SCE_PIN = OUT;
    TRAN_RESET_PIN = OUT;
    
    /* Assigning SDI1 and SDO1 to pins --> SCK1 cannot be reassigned (pin 50) */
    SDI1Rbits.SDI1R = 0b0000;    // RPD3 --> pin 51
    RPD5Rbits.RPD5R = 0b1000;    // RPD5 --> pin 53
    
    SPI1CON = 0;                 // clear all bits
    SPI1BUF;                     // clear the RX buffer by reading from it
    SPI1BRG = 1000;              // 1000 for 12kHz, 1 for 1MHz
    
    SPI1STATbits.SPIROV = 0;     // clear the overflow bit
    SPI1CONbits.MSTEN   = 1;     // make master
    SPI1CONbits.MSSEN   = 0;     // SS is *not* automatically driven
    SPI1CONbits.MODE16  = 0;     // 8-bit transfer mode
    SPI1CONbits.SMP     = 0;     // sample in the middle
    SPI1CONbits.CKE     = 1;     // output on falling edge
    SPI1CONbits.CKP     = 0;     // clock idle state is low
    
    asm volatile ("ei");
    
    SPI1CONbits.ON      = 1;    // turn on SPI 
}

void TranEnable(void){
    TRAN_SCE = LOW;
}

void TranDisable(void){
    TRAN_SCE = HIGH;
}

BYTE SPI_IO(BYTE o){
    SPI1BUF = o;
    while(!SPI1STATbits.SPIRBF);
    return SPI1BUF;
}