/* 
 * Author: Konrad Rozanski
 * Konrad's a pussy ;)
 */
#include "SX1276.h"

int packetIndex = 0;
int impHeaderMode = 0;

void wait_ms(int ms){
    unsigned int i, j;
    for (i=0; i<ms; i++){
        for (j=0; j<1300; j++) {}
    }
}

void wait_us(int us){
    unsigned int i, j;
    for (i=0; i<us; i++){
        for (j=0; j<13; j++) {}
    }
}

int begin(long frequency){
    TRAN_RESET_PIN = OUT;
    TRAN_RESET = LOW;
    wait_ms(10);
    TRAN_RESET = HIGH;
    wait_ms(10);

    BYTE version = read_register(REG_VERSION);
    if (version != 0x12) {
        return 0;
    }
    
    sleep();
    
    set_frequency(frequency);

    /* set base addresses */
    write_register(REG_FIFO_TX_BASE_ADDR, 0);
    write_register(REG_FIFO_RX_BASE_ADDR, 0);

    /* set LNA boost */
    write_register(REG_LNA, read_register(REG_LNA) | 0x03);

    /* set auto AGC */
    write_register(REG_MODEM_CONFIG_3, 0x04);

    /* set output power to 17 dBm */
    set_TX_power(17, PA_OUTPUT_PA_BOOST_PIN);

    /* put in standby mode */
    idle();

    return 1;
}

BYTE read_register(BYTE address){
    return single_transfer(address & 0x7f, 0x00);
}

void write_register(BYTE address, BYTE value){
    single_transfer(address | 0x80, value);
}

BYTE single_transfer(BYTE address, BYTE value){
    BYTE response;

    TranEnable();
    SPI_IO(address);              // send register that you want to read from first
    wait_us(15);
    response = SPI_IO(value);     // send dummy value so that slave can be given the clock
    TranDisable();
    wait_ms(1);
    
    return response;
}

void sleep(){
    write_register(REG_OP_MODE, MODE_LONG_RANGE_MODE | MODE_SLEEP);
}

void idle(){
    write_register(REG_OP_MODE, MODE_LONG_RANGE_MODE | MODE_STDBY);
}

void set_frequency(long frequency){
    //uint64_t frf = ((uint64_t)frequency << 19) / 32000000; 

    uint32_t frf = 14991360;    // for 915MHz
    
    write_register(REG_FRF_MSB, (uint8_t)(frf >> 16));
    write_register(REG_FRF_MID, (uint8_t)(frf >> 8));
    write_register(REG_FRF_LSB, (uint8_t)(frf >> 0));
}

void set_OCP(uint8_t mA){
    uint8_t OCPTrim = 27;

    if (mA <= 120) {
        OCPTrim = (mA - 45) / 5;
    } else if (mA <=240) {
        OCPTrim = (mA + 30) / 10;
    }
    write_register(REG_OCP, 0x20 | (0x1F & OCPTrim));
}

void set_TX_power(int level, int outputPin){
    if (PA_OUTPUT_RFO_PIN == outputPin) {
        // RFO
        if (level < 0) {
            level = 0;
        } else if (level > 14) {
            level = 14;
        }

        write_register(REG_PA_CONFIG, 0x70 | level);
        } else {
            if (level > 17) {
                if (level > 20) { level = 20; }
                level -= 3;                         // subtract 3 from level, so 18 - 20 maps to 15 - 17
                write_register(REG_PA_DAC, 0x87);   // High Power +20 dBm Operation 
                set_OCP(140);
            } else {
                if (level < 2) { level = 2; }
                write_register(REG_PA_DAC, 0x84);   //Default value PA_HF/LF or +17dBm
                set_OCP(100);
            }
            write_register(REG_PA_CONFIG, PA_BOOST | (level - 2));
    }
}

int beginPacket(int implicitHeader){
    if (isTransmitting()) {
        return 0;
    }

    // put in standby mode
    idle();

    if (implicitHeader) {
        implicitHeaderMode();
    } else {
        explicitHeaderMode();
    }

    // reset FIFO address and payload length
    write_register(REG_FIFO_ADDR_PTR, 0);
    write_register(REG_PAYLOAD_LENGTH, 0);

  return 1;
}

bool isTransmitting(){
  if ((read_register(REG_OP_MODE) & MODE_TX) == MODE_TX) {
    return true;
  }

  if (read_register(REG_IRQ_FLAGS) & IRQ_TX_DONE_MASK) {
    // clear IRQ's
    write_register(REG_IRQ_FLAGS, IRQ_TX_DONE_MASK);
  }
  return false;
}

void explicitHeaderMode(){
    impHeaderMode = 0;
    
    write_register(REG_MODEM_CONFIG_1, read_register(REG_MODEM_CONFIG_1) & 0xfe);
}

void implicitHeaderMode(){
    impHeaderMode = 1;
    
    write_register(REG_MODEM_CONFIG_1, read_register(REG_MODEM_CONFIG_1) | 0x01);
}

void sendString(char *string){
    while(*string != '\0'){
        writeByte(*string);
        string++;
    }
}

unsigned writeByte(uint8_t byte)
{
    return write(&byte, sizeof(byte));
}

unsigned write(const uint8_t *buffer, unsigned size)
{
  int currentLength = read_register(REG_PAYLOAD_LENGTH);

  // check size
  if ((currentLength + size) > MAX_PKT_LENGTH) {
    size = MAX_PKT_LENGTH - currentLength;
  }

  // write data
  for (unsigned i = 0; i < size; i++) {
    write_register(REG_FIFO, buffer[i]);
  }

  // update length
  write_register(REG_PAYLOAD_LENGTH, currentLength + size);

  return size;
}

int endPacket(bool async){
    // put in TX mode
    write_register(REG_OP_MODE, MODE_LONG_RANGE_MODE | MODE_TX);

    if (async) {
        // grace time is required for the radio
        wait_ms(150);
    } else {
        // wait for TX done
        while ((read_register(REG_IRQ_FLAGS) & IRQ_TX_DONE_MASK) == 0) {
        //yield();
    }
        // clear IRQ's
        write_register(REG_IRQ_FLAGS, IRQ_TX_DONE_MASK);
    }
    return 1;
}

/*
 * Receiving Code 
 */

int parse_packet(int size){
    int packetLength = 0;
    int irqFlags = read_register(REG_IRQ_FLAGS);

    if (size > 0) {
        implicitHeaderMode();
        write_register(REG_PAYLOAD_LENGTH, size & 0xff);
    } else {
        explicitHeaderMode();
    }

    // clear IRQ's
    write_register(REG_IRQ_FLAGS, irqFlags);

    if ((irqFlags & IRQ_RX_DONE_MASK) && (irqFlags & IRQ_PAYLOAD_CRC_ERROR_MASK) == 0) {
    
        // received a packet
        packetIndex = 0;

        // read packet length
        if (impHeaderMode) {
            packetLength = read_register(REG_PAYLOAD_LENGTH);
        } else {
            packetLength = read_register(REG_RX_NB_BYTES);
        }

        // set FIFO address to current RX address
        write_register(REG_FIFO_ADDR_PTR, read_register(REG_FIFO_RX_CURRENT_ADDR));

        // put in standby mode
        idle();
        } else if (read_register(REG_OP_MODE) != (MODE_LONG_RANGE_MODE | MODE_RX_SINGLE)) {
        // not currently in RX mode

        // reset FIFO address
        write_register(REG_FIFO_ADDR_PTR, 0);

        // put in single RX mode
        write_register(REG_OP_MODE, MODE_LONG_RANGE_MODE | MODE_RX_SINGLE);
    }
    return packetLength;
}

int available(){
    return (read_register(REG_RX_NB_BYTES) - packetIndex);
}

int read(){
    if (!available()) {
        return -1;
    }

    packetIndex++;

    return read_register(REG_FIFO);
}